<?php

/**
 * Create a new top-level menu.
 *
 * @return integer
 *   The database ID of the newly created menu
 */
function install_menu_create_menu($title, $weight = 0) {
  $mid = db_next_id('{menu}_mid');
  // Check explicitly for mid <= 2. If the database was improperly prefixed,
  // this would cause a nasty infinite loop or duplicate mid errors.
  // TODO: have automatic prefixing through an installer to prevent this.
  while ($mid <= 2) {
    $mid = db_next_id('{menu}_mid');
  }
  db_query("INSERT INTO {menu} (mid, pid, title, weight, type) VALUES (%d, 0, '%s', %d, 115)", $mid, $title, $weight);
  menu_rebuild(); // not sure if this is needed, but we've seen problems without it
  return $mid; // this is important to add new items to this menu.
}

/**
 * Get the menu ID, searching on path.
 *
 * @return integer
 *   The database ID of a menu item based on its path.
 */
function install_menu_get_mid($path) {
  menu_rebuild(); // not sure if this is needed, but we've seen problems without it
  return db_result(db_query_range("SELECT mid FROM {menu} WHERE path = '%s'", $path, 0, 1));	
}

/**
 * Get the menu ID of a root menu, based on title, e.g. Secondary links.
 *
 * @return integer
 *   The database ID of the root menu that matches the $title
 */
function install_menu_get_root_menu($title) {
  return db_result(db_query_range("SELECT mid FROM {menu} WHERE title = '%s'", $title, 0, 1));	
}

/**
 * Set an existing menu ID to a new parent.
 */
function install_menu_set_menu($mid, $pid, $weight = 0, $type = 54) {
  db_query("UPDATE {menu} SET pid = %d, type = %d, weight = %d WHERE mid = %d", $pid, $type, $weight, $mid);
  menu_rebuild(); // not sure if this is needed, but we've seen problems without it
}

/**
 * Create a new menu item.
 *
 * @param $path
 *   Path of the new menu item.
 * @param $title
 *   Title of the menu item (visible label for menu).
 * @param $pid
 *   Parent ID -- which menu the item is being added to.
 * @param $description
 *   Description of the menu item (tooltip).
 * @param $weight
 *   Weight for positioning.
 * @param $type
 *   Menu item type; new items are 118 by default.
 * @return integer
 *   The database ID of the menu item.
 */
function install_menu_create_menu_item($path, $title, $pid, $description = '', $weight = 0, $type = 118) {
  $menu = array(
    'path' => $path,
    'title' => $title,
    'pid' => $pid,
    'description' => $description,
    'weight' => $weight,
    'type' => $type
  );
  menu_save_item($menu);
  
  menu_rebuild(); // not sure if this is needed, but we've seen problems without it
  
  return install_menu_get_mid($path); // this is important to add new items to this menu.
}

/**
 * Creates a batch of menu items.
 *
 * @param $items
 *   An array containing menu items (as arrays).
 * @param $pid
 *   Parent ID -- which menu the items are being added to
 */
function install_menu_create_menu_items($items, $pid) {
  foreach ($items as $item) {
    $mid = install_menu_create_menu_item($item['path'], $item['title'], $pid, $item['description'], $item['weight'], $item['type']);
    if (isset($items['children'])) {
      foreach ($item['children'] as $children) {
        install_menu_create_menu_items($children, $mid);
      }
    }
  }
}

/**
 * Update a menu item.
 *
 * @param $mid
 *   Menu item id
 * @param $properties
 *   A keyed array of only the values you want to change.
 *
 * @return
 *   FALSE if menu does not exist, otherwise return the status returned by
 *   menu_save_item() in menu.module
 */
function install_menu_update_menu_item($mid, $properties) {
  $existing_item = menu_get_item($mid);
  if (empty($existing_item)) {
    drupal_set_message(t('Menu item '. $mid .' did not exist in install_menu_update_menu().'));
    return FALSE;
  }
  $edit = array_merge($existing_item, $properties);
  $edit['mid'] = $mid;
  // Tell menu this is an updated item.
  $edit['type'] = 54;

  $status = menu_save_item($edit);
  menu_rebuild();
  return $status;
}

/**
 * Remove the specified filter from the specified format.
 *
 * NOTE: the module name + the delta is what uniquely identifies a filter.
 *
 * @param $mid
 *   The ID of the menu item to disable.
 */
function install_menu_disable_item($mid) { 
  $item = menu_get_item($mid);
  $type = $item['type'];
  $type &= ~MENU_VISIBLE_IN_TREE;
  $type &= ~MENU_VISIBLE_IN_BREADCRUMB;
  $type |= MENU_MODIFIED_BY_ADMIN;
  db_query('UPDATE {menu} SET type = %d WHERE mid = %d', $type, $mid);
  drupal_set_message(t('The menu item has been disabled.'));
  // No redirection during install
  //drupal_goto('admin/build/menu');
}

